@@ -1,9 +1,12 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
 
 try:
     import isort.api
+    import isort.settings
 
     HAS_ISORT_5 = True
 except ImportError:  # isort < 5
@@ -11,17 +14,20 @@ except ImportError:  # isort < 5
 
     HAS_ISORT_5 = False
 
+import argparse
 import codecs
 import os
 import re
 import sys
 import textwrap
 import tokenize
+import warnings
+from collections.abc import Sequence
 from io import BufferedReader, BytesIO
 from typing import (
     TYPE_CHECKING,
+    Any,
     List,
-    Optional,
     Pattern,
     TextIO,
     Tuple,
@@ -33,6 +39,7 @@ from typing import (
 from astroid import Module, modutils, nodes
 
 from pylint.constants import PY_EXTS
+from pylint.typing import OptionDict
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -41,12 +48,12 @@ else:
 
 if TYPE_CHECKING:
     from pylint.checkers.base_checker import BaseChecker
+    from pylint.lint import PyLinter
 
 DEFAULT_LINE_LENGTH = 79
 
 # These are types used to overload get_global_option() and refer to the options type
 GLOBAL_OPTION_BOOL = Literal[
-    "ignore-mixin-members",
     "suggestion-mode",
     "analyse-fallback-blocks",
     "allow-global-unused-variables",
@@ -54,22 +61,35 @@ GLOBAL_OPTION_BOOL = Literal[
 GLOBAL_OPTION_INT = Literal["max-line-length", "docstring-min-length"]
 GLOBAL_OPTION_LIST = Literal["ignored-modules"]
 GLOBAL_OPTION_PATTERN = Literal[
-    "no-docstring-rgx", "dummy-variables-rgx", "ignored-argument-names"
+    "no-docstring-rgx",
+    "dummy-variables-rgx",
+    "ignored-argument-names",
+    "mixin-class-rgx",
 ]
+GLOBAL_OPTION_PATTERN_LIST = Literal["exclude-too-few-public-methods", "ignore-paths"]
 GLOBAL_OPTION_TUPLE_INT = Literal["py-version"]
 GLOBAL_OPTION_NAMES = Union[
     GLOBAL_OPTION_BOOL,
     GLOBAL_OPTION_INT,
     GLOBAL_OPTION_LIST,
     GLOBAL_OPTION_PATTERN,
+    GLOBAL_OPTION_PATTERN_LIST,
     GLOBAL_OPTION_TUPLE_INT,
 ]
 T_GlobalOptionReturnTypes = TypeVar(
-    "T_GlobalOptionReturnTypes", bool, int, List[str], Pattern, Tuple[int, ...]
+    "T_GlobalOptionReturnTypes",
+    bool,
+    int,
+    List[str],
+    Pattern[str],
+    List[Pattern[str]],
+    Tuple[int, ...],
 )
 
 
-def normalize_text(text, line_len=DEFAULT_LINE_LENGTH, indent=""):
+def normalize_text(
+    text: str, line_len: int = DEFAULT_LINE_LENGTH, indent: str = ""
+) -> str:
     """Wrap the text on the given line length."""
     return "\n".join(
         textwrap.wrap(
@@ -82,22 +102,22 @@ CMPS = ["=", "-", "+"]
 
 
 # py3k has no more cmp builtin
-def cmp(a, b):  # pylint: disable=redefined-builtin
+def cmp(a: int | float, b: int | float) -> int:
     return (a > b) - (a < b)
 
 
-def diff_string(old, new):
-    """given an old and new int value, return a string representing the
-    difference
+def diff_string(old: int | float, new: int | float) -> str:
+    """Given an old and new int value, return a string representing the
+    difference.
     """
     diff = abs(old - new)
     diff_str = f"{CMPS[cmp(old, new)]}{diff and f'{diff:.2f}' or ''}"
     return diff_str
 
 
-def get_module_and_frameid(node):
-    """return the module name and the frame id in the module"""
-    frame = node.frame()
+def get_module_and_frameid(node: nodes.NodeNG) -> tuple[str, str]:
+    """Return the module name and the frame id in the module."""
+    frame = node.frame(future=True)
     module, obj = "", []
     while frame:
         if isinstance(frame, Module):
@@ -105,20 +125,26 @@ def get_module_and_frameid(node):
         else:
             obj.append(getattr(frame, "name", "<lambda>"))
         try:
-            frame = frame.parent.frame()
+            frame = frame.parent.frame(future=True)
         except AttributeError:
             break
     obj.reverse()
     return module, ".".join(obj)
 
 
-def get_rst_title(title, character):
-    """Permit to get a title formatted as ReStructuredText test (underlined with a chosen character)."""
+def get_rst_title(title: str, character: str) -> str:
+    """Permit to get a title formatted as ReStructuredText test (underlined with a
+    chosen character).
+    """
     return f"{title}\n{character * len(title)}\n"
 
 
-def get_rst_section(section, options, doc=None):
-    """format an options section using as a ReStructuredText formatted output"""
+def get_rst_section(
+    section: str | None,
+    options: list[tuple[str, OptionDict, Any]],
+    doc: str | None = None,
+) -> str:
+    """Format an option's section using as a ReStructuredText formatted output."""
     result = ""
     if section:
         result += get_rst_title(section, "'")
@@ -129,24 +155,17 @@ def get_rst_section(section, options, doc=None):
         help_opt = optdict.get("help")
         result += f":{optname}:\n"
         if help_opt:
+            assert isinstance(help_opt, str)
             formatted_help = normalize_text(help_opt, indent="  ")
             result += f"{formatted_help}\n"
-        if value:
+        if value and optname != "py-version":
             value = str(_format_option_value(optdict, value))
             result += f"\n  Default: ``{value.replace('`` ', '```` ``')}``\n"
     return result
 
 
-def safe_decode(line, encoding, *args, **kwargs):
-    """return decoded line from encoding or decode with default encoding"""
-    try:
-        return line.decode(encoding or sys.getdefaultencoding(), *args, **kwargs)
-    except LookupError:
-        return line.decode(sys.getdefaultencoding(), *args, **kwargs)
-
-
 def decoding_stream(
-    stream: Union[BufferedReader, BytesIO],
+    stream: BufferedReader | BytesIO,
     encoding: str,
     errors: Literal["strict"] = "strict",
 ) -> codecs.StreamReader:
@@ -157,15 +176,15 @@ def decoding_stream(
     return reader_cls(stream, errors)
 
 
-def tokenize_module(node: nodes.Module) -> List[tokenize.TokenInfo]:
+def tokenize_module(node: nodes.Module) -> list[tokenize.TokenInfo]:
     with node.stream() as stream:
         readline = stream.readline
         return list(tokenize.tokenize(readline))
 
 
-def register_plugins(linter, directory):
-    """load all module and package in the given directory, looking for a
-    'register' function in each one, used to register pylint checkers
+def register_plugins(linter: PyLinter, directory: str) -> None:
+    """Load all module and package in the given directory, looking for a
+    'register' function in each one, used to register pylint checkers.
     """
     imported = {}
     for filename in os.listdir(directory):
@@ -186,7 +205,7 @@ def register_plugins(linter, directory):
                     os.path.join(directory, filename)
                 )
             except ValueError:
-                # empty module name (usually emacs auto-save files)
+                # empty module name (usually Emacs auto-save files)
                 continue
             except ImportError as exc:
                 print(f"Problem importing module {filename}: {exc}", file=sys.stderr)
@@ -198,94 +217,78 @@ def register_plugins(linter, directory):
 
 @overload
 def get_global_option(
-    checker: "BaseChecker", option: GLOBAL_OPTION_BOOL, default: Optional[bool] = None
+    checker: BaseChecker, option: GLOBAL_OPTION_BOOL, default: bool | None = ...
 ) -> bool:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker", option: GLOBAL_OPTION_INT, default: Optional[int] = None
+    checker: BaseChecker, option: GLOBAL_OPTION_INT, default: int | None = ...
 ) -> int:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_LIST,
-    default: Optional[List[str]] = None,
-) -> List[str]:
+    default: list[str] | None = ...,
+) -> list[str]:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_PATTERN,
-    default: Optional[Pattern] = None,
-) -> Pattern:
+    default: Pattern[str] | None = ...,
+) -> Pattern[str]:
     ...
 
 
 @overload
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
+    option: GLOBAL_OPTION_PATTERN_LIST,
+    default: list[Pattern[str]] | None = ...,
+) -> list[Pattern[str]]:
+    ...
+
+
+@overload
+def get_global_option(
+    checker: BaseChecker,
     option: GLOBAL_OPTION_TUPLE_INT,
-    default: Optional[Tuple[int, ...]] = None,
-) -> Tuple[int, ...]:
+    default: tuple[int, ...] | None = ...,
+) -> tuple[int, ...]:
     ...
 
 
 def get_global_option(
-    checker: "BaseChecker",
+    checker: BaseChecker,
     option: GLOBAL_OPTION_NAMES,
-    default: Optional[T_GlobalOptionReturnTypes] = None,
-) -> Optional[T_GlobalOptionReturnTypes]:
-    """Retrieve an option defined by the given *checker* or
+    default: T_GlobalOptionReturnTypes | None = None,  # pylint: disable=unused-argument
+) -> T_GlobalOptionReturnTypes | None | Any:
+    """DEPRECATED: Retrieve an option defined by the given *checker* or
     by all known option providers.
 
     It will look in the list of all options providers
     until the given *option* will be found.
     If the option wasn't found, the *default* value will be returned.
     """
-    # First, try in the given checker's config.
-    # After that, look in the options providers.
-
-    try:
-        return getattr(checker.config, option.replace("-", "_"))
-    except AttributeError:
-        pass
-    for provider in checker.linter.options_providers:
-        for options in provider.options:
-            if options[0] == option:
-                return getattr(provider.config, option.replace("-", "_"))
-    return default
+    warnings.warn(
+        "get_global_option has been deprecated. You can use "
+        "checker.linter.config to get all global options instead.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
+    return getattr(checker.linter.config, option.replace("-", "_"))
 
 
-def deprecated_option(
-    shortname=None, opt_type=None, help_msg=None, deprecation_msg=None
-):
-    def _warn_deprecated(option, optname, *args):  # pylint: disable=unused-argument
-        if deprecation_msg:
-            sys.stderr.write(deprecation_msg % (optname,))
-
-    option = {
-        "help": help_msg,
-        "hide": True,
-        "type": opt_type,
-        "action": "callback",
-        "callback": _warn_deprecated,
-        "deprecated": True,
-    }
-    if shortname:
-        option["shortname"] = shortname
-    return option
-
-
-def _splitstrip(string, sep=","):
-    """return a list of stripped string by splitting the string given as
-    argument on `sep` (',' by default). Empty string are discarded.
+def _splitstrip(string: str, sep: str = ",") -> list[str]:
+    """Return a list of stripped string by splitting the string given as
+    argument on `sep` (',' by default), empty strings are discarded.
 
     >>> _splitstrip('a, b, c   ,  4,,')
     ['a', 'b', 'c', '4']
@@ -306,13 +309,10 @@ def _splitstrip(string, sep=","):
     return [word.strip() for word in string.split(sep) if word.strip()]
 
 
-def _unquote(string):
-    """remove optional quotes (simple or double) from the string
+def _unquote(string: str) -> str:
+    """Remove optional quotes (simple or double) from the string.
 
-    :type string: str or unicode
     :param string: an optionally quoted string
-
-    :rtype: str or unicode
     :return: the unquoted string (or the input string if it wasn't quoted)
     """
     if not string:
@@ -324,20 +324,24 @@ def _unquote(string):
     return string
 
 
-def _check_csv(value):
+def _check_csv(value: list[str] | tuple[str] | str) -> Sequence[str]:
     if isinstance(value, (list, tuple)):
         return value
     return _splitstrip(value)
 
 
-def _comment(string):
-    """return string as a comment"""
+def _comment(string: str) -> str:
+    """Return string as a comment."""
     lines = [line.strip() for line in string.splitlines()]
-    return "# " + f"{os.linesep}# ".join(lines)
+    sep = "\n"
+    return "# " + f"{sep}# ".join(lines)
 
 
-def _format_option_value(optdict, value):
-    """return the user input's value from a 'compiled' value"""
+def _format_option_value(optdict: OptionDict, value: Any) -> str:
+    """Return the user input's value from a 'compiled' value.
+
+    TODO: 3.0: Remove deprecated function
+    """
     if optdict.get("type", None) == "py_version":
         value = ".".join(str(item) for item in value)
     elif isinstance(value, (list, tuple)):
@@ -351,31 +355,52 @@ def _format_option_value(optdict, value):
         value = "yes" if value else "no"
     elif isinstance(value, str) and value.isspace():
         value = f"'{value}'"
-    return value
+    return str(value)
 
 
 def format_section(
-    stream: TextIO, section: str, options: List[Tuple], doc: Optional[str] = None
+    stream: TextIO,
+    section: str,
+    options: list[tuple[str, OptionDict, Any]],
+    doc: str | None = None,
 ) -> None:
-    """format an options section using the INI format"""
+    """Format an option's section using the INI format."""
+    warnings.warn(
+        "format_section has been deprecated. It will be removed in pylint 3.0.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     if doc:
         print(_comment(doc), file=stream)
     print(f"[{section}]", file=stream)
-    _ini_format(stream, options)
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", category=DeprecationWarning)
+        _ini_format(stream, options)
 
 
-def _ini_format(stream: TextIO, options: List[Tuple]) -> None:
-    """format options using the INI format"""
+def _ini_format(stream: TextIO, options: list[tuple[str, OptionDict, Any]]) -> None:
+    """Format options using the INI format."""
+    warnings.warn(
+        "_ini_format has been deprecated. It will be removed in pylint 3.0.",
+        DeprecationWarning,
+        stacklevel=2,
+    )
     for optname, optdict, value in options:
+        # Skip deprecated option
+        if "kwargs" in optdict:
+            assert isinstance(optdict["kwargs"], dict)
+            if "new_names" in optdict["kwargs"]:
+                continue
         value = _format_option_value(optdict, value)
         help_opt = optdict.get("help")
         if help_opt:
+            assert isinstance(help_opt, str)
             help_opt = normalize_text(help_opt, indent="# ")
             print(file=stream)
             print(help_opt, file=stream)
         else:
             print(file=stream)
-        if value is None:
+        if value in {"None", "False"}:
             print(f"#{optname}=", file=stream)
         else:
             value = str(value).strip()
@@ -390,24 +415,25 @@ def _ini_format(stream: TextIO, options: List[Tuple]) -> None:
 class IsortDriver:
     """A wrapper around isort API that changed between versions 4 and 5."""
 
-    def __init__(self, config):
+    def __init__(self, config: argparse.Namespace) -> None:
         if HAS_ISORT_5:
-            self.isort5_config = isort.api.Config(
-                # There is not typo here. EXTRA_standard_library is
+            self.isort5_config = isort.settings.Config(
+                # There is no typo here. EXTRA_standard_library is
                 # what most users want. The option has been named
-                # KNOWN_standard_library for ages in pylint and we
+                # KNOWN_standard_library for ages in pylint, and we
                 # don't want to break compatibility.
                 extra_standard_library=config.known_standard_library,
                 known_third_party=config.known_third_party,
             )
         else:
-            self.isort4_obj = isort.SortImports(  # pylint: disable=no-member
+            # pylint: disable-next=no-member
+            self.isort4_obj = isort.SortImports(  # type: ignore[attr-defined]
                 file_contents="",
                 known_standard_library=config.known_standard_library,
                 known_third_party=config.known_third_party,
             )
 
-    def place_module(self, package):
+    def place_module(self, package: str) -> str:
         if HAS_ISORT_5:
             return isort.api.place_module(package, self.isort5_config)
-        return self.isort4_obj.place_module(package)
+        return self.isort4_obj.place_module(package)  # type: ignore[no-any-return]
