@@ -1,19 +1,6 @@
-# Copyright (c) 2014-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2017, 2020 Pedro Algarvio <pedro@algarvio.me>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2019 agutole <toldo_carp@hotmail.com>
-# Copyright (c) 2020 Ganden Schaffner <gschaffner@pm.me>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Eli Fine <ejfine@gmail.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
+# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 """Unittest for the spelling checker."""
 
@@ -21,7 +8,8 @@ import astroid
 import pytest
 
 from pylint.checkers import spelling
-from pylint.testutils import CheckerTestCase, Message, _tokenize_str, set_config
+from pylint.checkers.spelling import _get_enchant_dict_help
+from pylint.testutils import CheckerTestCase, MessageTest, _tokenize_str, set_config
 
 # try to create enchant dictionary
 try:
@@ -48,15 +36,29 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
         reason="missing python-enchant package or missing spelling dictionaries",
     )
 
-    def _get_msg_suggestions(self, word, count=4):
+    def _get_msg_suggestions(self, word: str, count: int = 4) -> str:
         suggestions = "' or '".join(self.checker.spelling_dict.suggest(word)[:count])
         return f"'{suggestions}'"
 
+    def test_spelling_dict_help_no_enchant(self) -> None:
+        assert "both the python package and the system dep" in _get_enchant_dict_help(
+            [], pyenchant_available=False
+        )
+        assert "need to install the system dep" in _get_enchant_dict_help(
+            [], pyenchant_available=True
+        )
+
+    @skip_on_missing_package_or_dict
+    def test_spelling_dict_help_enchant(self) -> None:
+        assert "Available dictionaries: " in _get_enchant_dict_help(
+            enchant.Broker().list_dicts(), pyenchant_available=True
+        )
+
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_check_bad_coment(self):
+    def test_check_bad_coment(self) -> None:
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -72,9 +74,9 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
     @set_config(max_spelling_suggestions=2)
-    def test_check_bad_coment_custom_suggestion_count(self):
+    def test_check_bad_comment_custom_suggestion_count(self) -> None:
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -89,10 +91,10 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_check_bad_docstring(self):
+    def test_check_bad_docstring(self) -> None:
         stmt = astroid.extract_node('def fff():\n   """bad coment"""\n   pass')
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -107,7 +109,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
         stmt = astroid.extract_node('class Abc(object):\n   """bad coment"""\n   pass')
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -120,67 +122,57 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
         ):
             self.checker.visit_classdef(stmt)
 
-    @pytest.mark.skipif(True, reason="pyenchant's tokenizer strips these")
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_invalid_docstring_characters(self):
-        stmt = astroid.extract_node('def fff():\n   """test\\x00"""\n   pass')
-        with self.assertAddsMessages(
-            Message("invalid-characters-in-docstring", line=2, args=("test\x00",))
-        ):
-            self.checker.visit_functiondef(stmt)
-
-    @skip_on_missing_package_or_dict
-    @set_config(spelling_dict=spell_dict)
-    def test_skip_shebangs(self):
+    def test_skip_shebangs(self) -> None:
         self.checker.process_tokens(_tokenize_str("#!/usr/bin/env python"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_python_coding_comments(self):
+    def test_skip_python_coding_comments(self) -> None:
         self.checker.process_tokens(_tokenize_str("# -*- coding: utf-8 -*-"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
         self.checker.process_tokens(_tokenize_str("# coding=utf-8"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
         self.checker.process_tokens(_tokenize_str("# vim: set fileencoding=utf-8 :"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
         # Now with a shebang first
         self.checker.process_tokens(
             _tokenize_str("#!/usr/bin/env python\n# -*- coding: utf-8 -*-")
         )
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
         self.checker.process_tokens(
             _tokenize_str("#!/usr/bin/env python\n# coding=utf-8")
         )
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
         self.checker.process_tokens(
             _tokenize_str("#!/usr/bin/env python\n# vim: set fileencoding=utf-8 :")
         )
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_top_level_pylint_enable_disable_comments(self):
+    def test_skip_top_level_pylint_enable_disable_comments(self) -> None:
         self.checker.process_tokens(
             _tokenize_str("# Line 1\n Line 2\n# pylint: disable=ungrouped-imports")
         )
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_words_with_numbers(self):
+    def test_skip_words_with_numbers(self) -> None:
         self.checker.process_tokens(_tokenize_str("\n# 0ne\n# Thr33\n# Sh3ll"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_wiki_words(self):
+    def test_skip_wiki_words(self) -> None:
         stmt = astroid.extract_node(
             'class ComentAbc(object):\n   """ComentAbc with a bad coment"""\n   pass'
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -195,12 +187,12 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_camel_cased_words(self):
+    def test_skip_camel_cased_words(self) -> None:
         stmt = astroid.extract_node(
             'class ComentAbc(object):\n   """comentAbc with a bad coment"""\n   pass'
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -218,7 +210,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
             'class ComentAbc(object):\n   """argumentN with a bad coment"""\n   pass'
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -243,37 +235,58 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
                 f'class TestClass(object):\n   """{ccn} comment"""\n   pass'
             )
             self.checker.visit_classdef(stmt)
-            assert self.linter.release_messages() == []
+            assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_words_with_underscores(self):
+    def test_skip_words_with_underscores(self) -> None:
         stmt = astroid.extract_node(
             'def fff(param_name):\n   """test param_name"""\n   pass'
         )
         self.checker.visit_functiondef(stmt)
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_email_address(self):
+    def test_skip_email_address(self) -> None:
         self.checker.process_tokens(_tokenize_str("# uname@domain.tld"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_urls(self):
+    def test_skip_urls(self) -> None:
         self.checker.process_tokens(_tokenize_str("# https://github.com/rfk/pyenchant"))
-        assert self.linter.release_messages() == []
+        assert not self.linter.release_messages()
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_sphinx_directives(self):
+    @pytest.mark.parametrize(
+        "type_comment",
+        [
+            "# type: (NotAWord) -> NotAWord",
+            "# type: List[NotAWord] -> List[NotAWord]",
+            "# type: Dict[NotAWord] -> Dict[NotAWord]",
+            "# type: NotAWord",
+            "# type: List[NotAWord]",
+            "# type: Dict[NotAWord]",
+            "# type: ImmutableList[Manager]",
+            # will result in error: Invalid "type: ignore" comment  [syntax]
+            # when analyzed with mypy 1.02
+            "# type: ignore[attr-defined] NotAWord",
+        ],
+    )
+    def test_skip_type_comments(self, type_comment: str) -> None:
+        self.checker.process_tokens(_tokenize_str(type_comment))
+        assert not self.linter.release_messages()
+
+    @skip_on_missing_package_or_dict
+    @set_config(spelling_dict=spell_dict)
+    def test_skip_sphinx_directives(self) -> None:
         stmt = astroid.extract_node(
             'class ComentAbc(object):\n   """This is :class:`ComentAbc` with a bad coment"""\n   pass'
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -288,12 +301,12 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_sphinx_directives_2(self):
+    def test_skip_sphinx_directives_2(self) -> None:
         stmt = astroid.extract_node(
             'class ComentAbc(object):\n   """This is :py:attr:`ComentAbc` with a bad coment"""\n   pass'
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -309,53 +322,38 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
     @pytest.mark.parametrize(
-        ",".join(
-            (
-                "misspelled_portion_of_directive",
-                "second_portion_of_directive",
-                "description",
-            )
-        ),
+        "prefix,suffix",
         (
-            ("fmt", ": on", "black directive to turn on formatting"),
-            ("fmt", ": off", "black directive to turn off formatting"),
-            ("noqa", "", "pycharm directive"),
-            ("noqa", ":", "flake8 / zimports directive"),
-            ("nosec", "", "bandit directive"),
-            ("isort", ":skip", "isort directive"),
-            ("mypy", ":", "mypy directive"),
+            pytest.param("fmt", ": on", id="black directive to turn on formatting"),
+            pytest.param("fmt", ": off", id="black directive to turn off formatting"),
+            pytest.param("noqa", "", id="pycharm directive"),
+            pytest.param("noqa", ":", id="flake8 / zimports directive"),
+            pytest.param("nosec", "", id="bandit directive"),
+            pytest.param("isort", ":skip", id="isort directive"),
+            pytest.param("mypy", ":", id="mypy top of file directive"),
         ),
     )
-    def test_skip_tool_directives_at_beginning_of_comments_but_still_raise_error_if_directive_appears_later_in_comment(  # pylint:disable=unused-argument
-        # Having the extra description parameter allows the description
-        #   to show up in the pytest output as part of the test name
-        #   when running parametrized tests.
-        self,
-        misspelled_portion_of_directive,
-        second_portion_of_directive,
-        description,
-    ):
-        full_comment = f"# {misspelled_portion_of_directive}{second_portion_of_directive} {misspelled_portion_of_directive}"
+    def test_tool_directives_handling(self, prefix: str, suffix: str) -> None:
+        """We're not raising when the directive is at the beginning of comments,
+        but we raise if a directive appears later in comment."""
+        full_comment = f"# {prefix}{suffix} {prefix}"
+        args = (
+            prefix,
+            full_comment,
+            f"  {'^' * len(prefix)}",
+            self._get_msg_suggestions(prefix),
+        )
         with self.assertAddsMessages(
-            Message(
-                "wrong-spelling-in-comment",
-                line=1,
-                args=(
-                    misspelled_portion_of_directive,
-                    full_comment,
-                    f"  {'^'*len(misspelled_portion_of_directive)}",
-                    self._get_msg_suggestions(misspelled_portion_of_directive),
-                ),
-            )
+            MessageTest("wrong-spelling-in-comment", line=1, args=args)
         ):
             self.checker.process_tokens(_tokenize_str(full_comment))
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_code_flanked_in_double_backticks(self):
+    def test_skip_code_flanked_in_double_backticks(self) -> None:
         full_comment = "# The function ``.qsize()`` .qsize()"
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -370,10 +368,10 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_skip_code_flanked_in_single_backticks(self):
+    def test_skip_code_flanked_in_single_backticks(self) -> None:
         full_comment = "# The function `.qsize()` .qsize()"
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -391,10 +389,10 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
         spelling_dict=spell_dict,
         spelling_ignore_comment_directives="newdirective:,noqa",
     )
-    def test_skip_directives_specified_in_pylintrc(self):
+    def test_skip_directives_specified_in_pylintrc(self) -> None:
         full_comment = "# newdirective: do this newdirective"
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -409,7 +407,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_handle_words_joined_by_forward_slash(self):
+    def test_handle_words_joined_by_forward_slash(self) -> None:
         stmt = astroid.extract_node(
             '''
         class ComentAbc(object):
@@ -418,7 +416,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
         '''
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=3,
                 args=(
@@ -433,12 +431,12 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_more_than_one_error_in_same_line_for_same_word_on_docstring(self):
+    def test_more_than_one_error_in_same_line_for_same_word_on_docstring(self) -> None:
         stmt = astroid.extract_node(
             'class ComentAbc(object):\n   """Check teh dummy comment teh"""\n   pass'
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -448,7 +446,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
                     self._get_msg_suggestions("teh"),
                 ),
             ),
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -463,9 +461,9 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_more_than_one_error_in_same_line_for_same_word_on_comment(self):
+    def test_more_than_one_error_in_same_line_for_same_word_on_comment(self) -> None:
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -475,7 +473,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
                     self._get_msg_suggestions("coment"),
                 ),
             ),
-            Message(
+            MessageTest(
                 "wrong-spelling-in-comment",
                 line=1,
                 args=(
@@ -490,7 +488,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_docstring_lines_that_look_like_comments_1(self):
+    def test_docstring_lines_that_look_like_comments_1(self) -> None:
         stmt = astroid.extract_node(
             '''def f():
     """
@@ -498,7 +496,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
     """'''
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=3,
                 args=(
@@ -513,13 +511,13 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_docstring_lines_that_look_like_comments_2(self):
+    def test_docstring_lines_that_look_like_comments_2(self) -> None:
         stmt = astroid.extract_node(
             '''def f():
     """# msitake"""'''
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=2,
                 args=(
@@ -534,7 +532,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_docstring_lines_that_look_like_comments_3(self):
+    def test_docstring_lines_that_look_like_comments_3(self) -> None:
         stmt = astroid.extract_node(
             '''def f():
     """
@@ -542,7 +540,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
     """'''
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=3,
                 args=(
@@ -557,7 +555,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_docstring_lines_that_look_like_comments_4(self):
+    def test_docstring_lines_that_look_like_comments_4(self) -> None:
         stmt = astroid.extract_node(
             '''def f():
     """
@@ -569,7 +567,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_docstring_lines_that_look_like_comments_5(self):
+    def test_docstring_lines_that_look_like_comments_5(self) -> None:
         stmt = astroid.extract_node(
             '''def f():
     """
@@ -577,7 +575,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
     """'''
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=3,
                 args=(
@@ -592,7 +590,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
 
     @skip_on_missing_package_or_dict
     @set_config(spelling_dict=spell_dict)
-    def test_docstring_lines_that_look_like_comments_6(self):
+    def test_docstring_lines_that_look_like_comments_6(self) -> None:
         stmt = astroid.extract_node(
             '''def f():
     """
@@ -600,7 +598,7 @@ class TestSpellingChecker(CheckerTestCase):  # pylint:disable=too-many-public-me
     """'''
         )
         with self.assertAddsMessages(
-            Message(
+            MessageTest(
                 "wrong-spelling-in-docstring",
                 line=3,
                 args=(
